"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompoundVFS = void 0;
/**
 * Combine multiple VFSs into a single compound one.
 *
 * Files are resolved in order - the first VFS to return a file wins.
 */
class CompoundVFS {
    constructor(fileSystems) {
        this.fileSystems = fileSystems || [];
    }
    info(path) {
        for (let fs of this.fileSystems) {
            let info = fs.info(path);
            if (info != null) {
                return info;
            }
        }
        return null;
    }
    listFiles() {
        let result = [];
        for (let fs of this.fileSystems) {
            result.push(...fs.listFiles());
        }
        return result;
    }
    read(path) {
        for (let fs of this.fileSystems) {
            let data = fs.read(path);
            if (data != null) {
                return data;
            }
        }
        return null;
    }
    getTargetsFor(source) {
        let targets = null;
        for (let transform of this.fileSystems) {
            const transformTargets = transform.getTargetsFor(source);
            if (transformTargets) {
                if (targets == null) {
                    targets = [];
                }
                targets.push(...transformTargets);
            }
        }
        return targets;
    }
}
exports.CompoundVFS = CompoundVFS;
//# sourceMappingURL=CompoundVFS.js.map