"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SchemaTransform = void 0;
const SingleTransform_1 = require("./SingleTransform");
const core_1 = require("@journeyapps/core");
const parser_schema_1 = require("@journeyapps/parser-schema");
const parser_common_1 = require("@journeyapps/parser-common");
const schemaToDefinitions_1 = require("./schemaToDefinitions");
const caching_1 = require("./caching");
class SchemaTransform extends SingleTransform_1.SingleTransform {
    constructor(fs, options) {
        super({
            inputs: [options.sourcePath, options.applicationXmlPath],
            target: options.definitionPath,
            isScript: true
        });
        this.schemaCache = null;
        this.generateSingle = (0, caching_1.cachedSingleGenerate)({
            version: () => this.getVersion(),
            generate: () => {
                const text = this.fs.readTextSync(this.options.sourcePath);
                const applicationXml = this.fs.readTextSync(this.options.applicationXmlPath);
                const appInfo = core_1.applicationParser.parse(applicationXml);
                const schema = new parser_schema_1.Schema();
                schema.loadXml(text, { apiVersion: new parser_common_1.Version('4.0'), recordSource: true });
                this.schemaCache = schema;
                return (0, schemaToDefinitions_1.schemaToDefinitions)(schema, {
                    sourcePath: this.options.sourcePath,
                    asynchronous: this.options.asynchronous,
                    dbs: this.options.dbs,
                    mobileUserType: this.options.includeUser ? appInfo.mobileUserType : undefined
                });
            }
        });
        this.fs = fs;
        this.options = options;
    }
    loadSchema() {
        // Populate cache
        this.generateSingle();
        return this.schemaCache;
    }
    getVersion() {
        return (0, caching_1.getInputsVersions)(this.fs, this.inputs);
    }
}
exports.SchemaTransform = SchemaTransform;
//# sourceMappingURL=SchemaTransform.js.map