import { SourceNode } from '@journeyapps/source-utils';
import { VFS } from './VFS';
import { SourceEntry } from './SourceEntry';

/**
 * Combine multiple VFSs into a single compound one.
 *
 * Files are resolved in order - the first VFS to return a file wins.
 */
export class CompoundVFS implements VFS {
  fileSystems: VFS[];

  constructor(fileSystems?: VFS[]) {
    this.fileSystems = fileSystems || [];
  }

  info(path: string): SourceEntry | null {
    for (let fs of this.fileSystems) {
      let info = fs.info(path);
      if (info != null) {
        return info;
      }
    }
    return null;
  }

  listFiles(): string[] {
    let result: string[] = [];
    for (let fs of this.fileSystems) {
      result.push(...fs.listFiles());
    }
    return result;
  }

  read(path: string): SourceNode | null {
    for (let fs of this.fileSystems) {
      let data = fs.read(path);
      if (data != null) {
        return data;
      }
    }
    return null;
  }

  getTargetsFor(source: string): string[] | null {
    let targets: string[] | null = null;
    for (let transform of this.fileSystems) {
      const transformTargets = transform.getTargetsFor(source);
      if (transformTargets) {
        if (targets == null) {
          targets = [];
        }
        targets.push(...transformTargets);
      }
    }
    return targets;
  }
}
