import { SourceNode, SyncFileSystem } from '@journeyapps/source-utils';

export function getInputsVersions(fs: SyncFileSystem, inputs: string[]) {
  let versions: string[] = [];
  for (let input of inputs) {
    const version = fs.getVersion(input);
    if (version == null) {
      return null;
    }
    versions.push(`${input}:${version}`);
  }
  return versions.join(':');
}

interface CacheEntry {
  version: string;
  data: SourceNode | null;
}

export function cachedGenerate(options: {
  fs: SyncFileSystem;
  generate(target: string): SourceNode | null;
  getInputsFor(target: string): string[] | null;
}) {
  const cache: Record<string, CacheEntry> = {};

  return (target: string): SourceNode | null => {
    const inputs = options.getInputsFor(target);
    if (inputs == null) {
      return null;
    }
    const version = getInputsVersions(options.fs, inputs);
    if (version == null) {
      return null;
    }

    if (cache[target]?.version == version) {
      return cache[target].data;
    }
    const generated = options.generate(target);
    cache[target] = { version, data: generated };

    return generated;
  };
}

export function cachedSingleGenerate(options: { version(): string | null; generate(): SourceNode | null }) {
  let cache: CacheEntry | null = null;

  return (): SourceNode | null => {
    const version = options.version();
    if (version == null) {
      return null;
    }

    if (cache?.version == version) {
      return cache.data;
    }
    const generated = options.generate();
    cache = { version, data: generated };

    return generated;
  };
}
